@@ -65,6 +65,7 @@ extern "C" {
 PtDspyError DspyReorderFormatting(int formatCount, PtDspyDevFormat *format, int outFormatCount, const PtDspyDevFormat *outFormat);
 PtDspyError DspyFindStringInParamList(const char *string, char **result, int n, const UserParameter *p);
 PtDspyError DspyFindIntInParamList(const char *string, int *result, int n, const UserParameter *p);
+PtDspyError DspyFindFloatInParamList(const char *string, float *result, int n, const UserParameter *p);
 PtDspyError DspyFindMatrixInParamList(const char *string, float *result, int n, const UserParameter *p);
 PtDspyError DspyFindIntsInParamList(const char *string, int *resultCount, int *result, int n, const UserParameter *p);
 #ifdef __cplusplus
@@ -100,7 +101,7 @@ void SaveAsShadowMap(const std::string& filename, SqDisplayInstance* image)
 	#else
 			sprintf( version, "%s %s", STRNAME, VERSION );
 	#endif
-			TIFFSetField( pshadow, TIFFTAG_SOFTWARE, ( uint32 ) version );
+			TIFFSetField( pshadow, TIFFTAG_SOFTWARE, ( char* ) version );
 			TIFFSetField( pshadow, TIFFTAG_PIXAR_MATRIX_WORLDTOCAMERA, image->m_matWorldToCamera );
 			TIFFSetField( pshadow, TIFFTAG_PIXAR_MATRIX_WORLDTOSCREEN, image->m_matWorldToScreen );
 			TIFFSetField( pshadow, TIFFTAG_PIXAR_TEXTUREFORMAT, SHADOWMAP_HEADER );
@@ -112,7 +113,7 @@ void SaveAsShadowMap(const std::string& filename, SqDisplayInstance* image)
 		#else
 			sprintf( version, "%s %s", STRNAME, VERSION );
 		#endif
-			TIFFSetField( pshadow, TIFFTAG_SOFTWARE, ( uint32 ) version );
+			TIFFSetField( pshadow, TIFFTAG_SOFTWARE, ( char* ) version );
 			TIFFSetField( pshadow, TIFFTAG_IMAGEWIDTH, image->m_width );
 			TIFFSetField( pshadow, TIFFTAG_IMAGELENGTH, image->m_height );
 			TIFFSetField( pshadow, TIFFTAG_PLANARCONFIG, PLANARCONFIG_CONTIG );
@@ -224,7 +225,7 @@ void WriteTIFF(const std::string& filename, SqDisplayInstance* image)
 
         bool use_logluv = false;
 
-        TIFFSetField( pOut, TIFFTAG_SOFTWARE, ( uint32 ) version );
+        TIFFSetField( pOut, TIFFTAG_SOFTWARE, ( char* ) version );
         TIFFSetField( pOut, TIFFTAG_IMAGEWIDTH, ( uint32 ) image->m_width );
         TIFFSetField( pOut, TIFFTAG_IMAGELENGTH, ( uint32 ) image->m_height );
         TIFFSetField( pOut, TIFFTAG_ORIENTATION, ORIENTATION_TOPLEFT );
@@ -349,11 +350,25 @@ PtDspyError DspyImageOpen(PtDspyImageHandle * image,
 		if(strcmp(drivername, "file")==0 || strcmp(drivername, "tiff")==0)
 			pImage->m_imageType = Type_File;
 		else if(strcmp(drivername, "framebuffer")==0)
+#ifndef AQSIS_SYSTEM_MACOSX
 			pImage->m_imageType = Type_Framebuffer;
+#else
+        {
+            std::cout << "framebuffer display type not supported under MacOSX, reverting to 'file'" << std::endl;
+            pImage->m_imageType = Type_File;
+        }
+#endif
 		else if(strcmp(drivername, "zfile")==0)
 			pImage->m_imageType = Type_ZFile;
 		else if(strcmp(drivername, "zframebuffer")==0)
+#ifndef AQSIS_SYSTEM_MACOSX
 			pImage->m_imageType = Type_ZFramebuffer;
+#else
+        {
+            std::cout << "zframebuffer display type not supported under MacOSX, reverting to 'zfile'" << std::endl;
+            pImage->m_imageType = Type_ZFile;
+        }
+#endif
 		else if(strcmp(drivername, "shadow")==0)
 			pImage->m_imageType = Type_Shadowmap;
 		pImage->m_iFormatCount = iFormatCount;
@@ -402,6 +417,7 @@ PtDspyError DspyImageOpen(PtDspyImageHandle * image,
 			}
 			widestFormat = PkDspyUnsigned8;
 
+#ifndef AQSIS_SYSTEM_MACOSX
 			pImage->m_theWindow = new Fl_Window(pImage->m_width, pImage->m_height);
 			pImage->m_uiImageWidget = new Fl_FrameBuffer_Widget(0,0, pImage->m_width, pImage->m_height, pImage->m_iFormatCount, reinterpret_cast<unsigned char*>(pImage->m_data));
 			pImage->m_theWindow->resizable(pImage->m_uiImageWidget);
@@ -409,6 +425,7 @@ PtDspyError DspyImageOpen(PtDspyImageHandle * image,
 			pImage->m_theWindow->end();
 			Fl::visual(FL_RGB);
 			pImage->m_theWindow->show();
+#endif
 		}
         else
 		{
@@ -440,6 +457,7 @@ PtDspyError DspyImageOpen(PtDspyImageHandle * image,
 		// If in "zframebuffer" mode, we need another buffer for the displayed depth data.
 		if(pImage->m_imageType == Type_ZFramebuffer)
 		{
+#ifndef AQSIS_SYSTEM_MACOSX
 			pImage->m_zfbdata = reinterpret_cast<unsigned char*>(malloc( pImage->m_width * pImage->m_height * 3 * sizeof(unsigned char)));
 
 			pImage->m_theWindow = new Fl_Window(pImage->m_width, pImage->m_height);
@@ -449,6 +467,7 @@ PtDspyError DspyImageOpen(PtDspyImageHandle * image,
 			pImage->m_theWindow->end();
 			Fl::visual(FL_RGB);
 			pImage->m_theWindow->show();
+#endif
 		}
 
 		// If we are recieving "rgba" data, ensure that it is in the correct order.
@@ -490,6 +509,11 @@ PtDspyError DspyImageOpen(PtDspyImageHandle * image,
 		TqInt count = 2;
 		DspyFindIntsInParamList("origin", &count, pImage->m_origin, paramCount, parameters);
 		DspyFindIntsInParamList("OriginalSize", &count, pImage->m_OriginalSize, paramCount, parameters);
+
+		// Determine if we are creating a combined shadow map for ambient occlusion.
+		float append;
+		if( DspyFindFloatInParamList("append", &append, paramCount, parameters ) == PkDspyErrorNone )
+			pImage->m_append = (append != 0.0f)? 1:0;
 	}
 	else
 		return(PkDspyErrorNoMemory);
@@ -551,13 +575,13 @@ PtDspyError DspyImageData(PtDspyImageHandle image,
 					if( alpha > 0 )
 					{
 						TqInt so = ( y * pImage->m_lineLength ) + ( x * pImage->m_entrySize );
-						unsigned char r = _pdatarow[0];
-						unsigned char g = _pdatarow[1];
-						unsigned char b = _pdatarow[2];
+						int r = _pdatarow[0];
+						int g = _pdatarow[1];
+						int b = _pdatarow[2];
 						// C = INT_PRELERP( A, B, b, t )
-						unsigned char R = static_cast<int>(INT_PRELERP( reinterpret_cast<unsigned char*>(pImage->m_data)[ so + 0 ], r, alpha, t ));
-						unsigned char G = static_cast<int>(INT_PRELERP( reinterpret_cast<unsigned char*>(pImage->m_data)[ so + 1 ], g, alpha, t ));
-						unsigned char B = static_cast<int>(INT_PRELERP( reinterpret_cast<unsigned char*>(pImage->m_data)[ so + 2 ], b, alpha, t ));
+						int R = static_cast<int>(INT_PRELERP( static_cast<int>(reinterpret_cast<unsigned char*>(pImage->m_data)[ so + 0 ]), r, alpha, t ));
+						int G = static_cast<int>(INT_PRELERP( static_cast<int>(reinterpret_cast<unsigned char*>(pImage->m_data)[ so + 1 ]), g, alpha, t ));
+						int B = static_cast<int>(INT_PRELERP( static_cast<int>(reinterpret_cast<unsigned char*>(pImage->m_data)[ so + 2 ]), b, alpha, t ));
 						reinterpret_cast<unsigned char*>(pImage->m_data)[ so + 0 ] = CLAMP( R, 0, 255 );
 						reinterpret_cast<unsigned char*>(pImage->m_data)[ so + 1 ] = CLAMP( G, 0, 255 );
 						reinterpret_cast<unsigned char*>(pImage->m_data)[ so + 2 ] = CLAMP( B, 0, 255 );
@@ -630,6 +654,7 @@ PtDspyError DspyImageDelayClose(PtDspyImageHandle image)
 
 	if(pImage)
 	{
+#ifndef AQSIS_SYSTEM_MACOSX
 		if(pImage->m_imageType == Type_Framebuffer || pImage->m_imageType == Type_ZFramebuffer)
 		{
 			if( pImage->m_imageType == Type_ZFramebuffer )
@@ -693,6 +718,7 @@ PtDspyError DspyImageDelayClose(PtDspyImageHandle image)
 			}
 			Fl::run();
 		}
+#endif
 		return(DspyImageClose(image));
 	}
 	return(PkDspyErrorNone);
