@@ -36,6 +36,7 @@
 
 #include	"imagers.h"
 
+#include	<algorithm>
 
 
 
@@ -49,14 +50,20 @@ START_NAMESPACE( Aqsis )
 
 TqInt	CqBucket::m_XSize;
 TqInt	CqBucket::m_YSize;
-TqInt	CqBucket::m_FilterXWidth;
-TqInt	CqBucket::m_FilterYWidth;
-TqInt	CqBucket::m_XMax;
-TqInt	CqBucket::m_YMax;
+TqInt	CqBucket::m_RealWidth;
+TqInt	CqBucket::m_RealHeight;
+TqInt	CqBucket::m_DiscreteShiftX;
+TqInt	CqBucket::m_DiscreteShiftY;
 TqInt	CqBucket::m_XOrigin;
 TqInt	CqBucket::m_YOrigin;
 TqInt	CqBucket::m_PixelXSamples;
 TqInt	CqBucket::m_PixelYSamples;
+TqFloat	CqBucket::m_FilterXWidth;
+TqFloat	CqBucket::m_FilterYWidth;
+TqInt	CqBucket::m_NumTimeRanges;
+TqInt	CqBucket::m_NumDofBounds;
+CqImageBuffer* CqBucket::m_ImageBuffer;
+std::vector<CqBound>		CqBucket::m_DofBounds;
 std::vector<CqImagePixel>	CqBucket::m_aieImage;
 std::vector<std::vector<CqVector2D> >	CqBucket::m_aSamplePositions;
 std::vector<TqFloat> CqBucket::m_aFilterValues;
@@ -69,37 +76,39 @@ std::vector<TqFloat> CqBucket::m_aCoverages;
  *  Clear,Allocate, Init. the m_aieImage samples
  */
 
-void CqBucket::InitialiseBucket( TqInt xorigin, TqInt yorigin, TqInt xsize, TqInt ysize, TqInt xfwidth, TqInt yfwidth, TqInt xsamples, TqInt ysamples, TqBool fJitter )
+void CqBucket::InitialiseBucket( TqInt xorigin, TqInt yorigin, TqInt xsize, TqInt ysize, TqBool fJitter, TqBool empty )
 {
     m_XOrigin = xorigin;
     m_YOrigin = yorigin;
     m_XSize = xsize;
     m_YSize = ysize;
-    m_FilterXWidth = xfwidth;
-    m_FilterYWidth = yfwidth;
-    m_XMax = static_cast<TqInt>( CEIL( ( xfwidth - 1 ) * 0.5f ) );
-    m_YMax = static_cast<TqInt>( CEIL( ( xfwidth - 1 ) * 0.5f ) );
-    m_PixelXSamples = xsamples;
-    m_PixelYSamples = ysamples;
+	m_PixelXSamples = m_ImageBuffer->PixelXSamples();
+	m_PixelYSamples = m_ImageBuffer->PixelYSamples();
+	m_FilterXWidth = m_ImageBuffer->FilterXWidth();
+	m_FilterYWidth = m_ImageBuffer->FilterYWidth();
+	m_DiscreteShiftX = FLOOR(m_FilterXWidth/2.0f);
+	m_DiscreteShiftY = FLOOR(m_FilterYWidth/2.0f);
+	m_RealWidth = m_XSize + (m_DiscreteShiftX*2);
+	m_RealHeight = m_YSize + (m_DiscreteShiftY*2);
 
-    TqInt ywidth1, xwidth1;
-    ywidth1 = m_YSize + m_FilterYWidth;
-    xwidth1 = m_XSize + m_FilterXWidth;
+	m_NumTimeRanges = MAX(4, m_PixelXSamples * m_PixelYSamples);
 
     // Allocate the image element storage if this is the first bucket
     if(m_aieImage.empty())
     {
-        m_aieImage.resize( xwidth1 * ywidth1);
-        m_aSamplePositions.resize( xwidth1 * ywidth1 );
+        m_aieImage.resize( m_RealWidth * m_RealHeight );
+		m_aSamplePositions.resize( m_RealWidth * m_RealHeight );
+
+		CalculateDofBounds();
 
         // Initialise the samples for this bucket.
         TqInt which = 0;
-        for ( TqInt i = 0; i < ywidth1; i++ )
+        for ( TqInt i = 0; i < m_RealHeight; i++ )
         {
-            for ( TqInt j = 0; j < xwidth1; j++ )
+            for ( TqInt j = 0; j < m_RealWidth; j++ )
             {
                 m_aieImage[which].Clear();
-                m_aieImage[which].AllocateSamples( xsamples, ysamples );
+                m_aieImage[which].AllocateSamples( m_PixelXSamples, m_PixelYSamples );
                 m_aieImage[which].InitialiseSamples( m_aSamplePositions[which], fJitter );
 
                 which++;
@@ -107,30 +116,92 @@ void CqBucket::InitialiseBucket( TqInt xorigin, TqInt yorigin, TqInt xsize, TqIn
         }
     }
 
-    // now shuffle the pixels around and add in the pixel offset to the position.
-    static CqRandom random(  53 );
-    TqInt shuffleX = random.RandomInt( xwidth1 );
-    TqInt shuffleY = random.RandomInt( ywidth1 );
-    TqInt which = 0;
-    TqInt sourceIndex = shuffleY*xwidth1 + shuffleX;
-    TqInt numPixels = xwidth1*ywidth1;
-    for ( TqInt i = 0; i < ywidth1; i++ )
-    {
-        for ( TqInt j = 0; j < xwidth1; j++ )
-        {
-            CqVector2D bPos2( m_XOrigin, m_YOrigin );
-            bPos2 += CqVector2D( ( j - m_FilterXWidth / 2 ), ( i - m_FilterYWidth / 2 ) );
+	//:TODO: AGG, sort out sample shuffling.
+	// now shuffle the pixels around and add in the pixel offset to the position.
+//	std::random_shuffle(m_aImageIndex.begin(), m_aImageIndex.end());
 
-            m_aieImage[which].Clear();
-            m_aieImage[which].OffsetSamples( bPos2, m_aSamplePositions[sourceIndex] );
+	TqInt which = 0;
+	TqInt numPixels = m_RealWidth*m_RealHeight;
+	for ( TqInt i = 0; i < m_RealHeight; i++ )
+	{
+		for ( TqInt j = 0; j < m_RealWidth; j++ )
+		{
+			CqVector2D bPos2( m_XOrigin, m_YOrigin );
+			bPos2 += CqVector2D( ( j - m_DiscreteShiftX ), ( i - m_DiscreteShiftY ) );
 
-            which++;
-            sourceIndex = (sourceIndex+1) % (numPixels);
-        }
-    }
+			if(!empty)
+				m_aieImage[which].Clear();
+
+			m_aieImage[which].OffsetSamples( bPos2, m_aSamplePositions[which] );
+
+			which++;
+		}
+	}
 }
 
 
+void CqBucket::CalculateDofBounds()
+{
+	m_NumDofBounds = m_PixelXSamples * m_PixelYSamples;
+	m_DofBounds.resize(m_NumDofBounds);
+
+	TqFloat dx = 2.0 / m_PixelXSamples;
+	TqFloat dy = 2.0 / m_PixelYSamples;
+
+	// I know this is far from an optimal way of calculating this,
+	// but it's only done once so I don't care.
+	// Calculate the bounding boxes that the dof offset positions fall into.
+	TqFloat minX = -1.0;
+	TqFloat minY = -1.0;
+	TqInt which = 0;
+	for(int j = 0; j < m_PixelYSamples; ++j)
+	{
+		for(int i = 0; i < m_PixelXSamples; ++i)
+		{
+			CqVector2D topLeft(minX, minY);
+			CqVector2D topRight(minX + dx, minY);
+			CqVector2D bottomLeft(minX, minY + dy);
+			CqVector2D bottomRight(minX + dx, minY + dy);
+
+			CqImagePixel::ProjectToCircle(topLeft);
+			CqImagePixel::ProjectToCircle(topRight);
+			CqImagePixel::ProjectToCircle(bottomLeft);
+			CqImagePixel::ProjectToCircle(bottomRight);
+
+			// if the bound straddles x=0 or y=0 then just using the corners
+			// will give too small a bound, so we enlarge it by including the
+			// non-projected coords.
+			if((topLeft.y() > 0.0 && bottomLeft.y() < 0.0) ||
+				(topLeft.y() < 0.0 && bottomLeft.y() > 0.0))
+			{
+				topLeft.x(minX);
+				bottomLeft.x(minX);
+				topRight.x(minX + dx);
+				bottomRight.x(minX + dx);
+			}
+			if((topLeft.x() > 0.0 && topRight.x() < 0.0) ||
+				(topLeft.x() < 0.0 && topRight.x() > 0.0))
+			{
+				topLeft.y(minY);
+				bottomLeft.y(minY + dy);
+				topRight.y(minY);
+				bottomRight.y(minY + dy);
+			}
+
+			m_DofBounds[which].vecMin() = topLeft;
+			m_DofBounds[which].vecMax() = topLeft;
+			m_DofBounds[which].Encapsulate(topRight);
+			m_DofBounds[which].Encapsulate(bottomLeft);
+			m_DofBounds[which].Encapsulate(bottomRight);
+
+			which++;
+			minX += dx;
+		}
+		minX = -1.0;
+		minY += dy;
+	}
+}
+
 //----------------------------------------------------------------------
 /** Initialise the static filter values.
  */
@@ -141,10 +212,10 @@ void CqBucket::InitialiseFilterValues()
         return;
 
     // Allocate and fill in the filter values array for each pixel.
-    TqInt numsubpixels = ( m_PixelXSamples * m_PixelYSamples );
+    TqInt numsubpixels = ( PixelXSamples() * PixelYSamples() );
     TqInt numperpixel = numsubpixels * numsubpixels;
 
-    TqUint numvalues = static_cast<TqUint>( ( ( m_FilterXWidth + 1 ) * ( m_FilterYWidth + 1 ) ) * ( numperpixel ) );
+    TqUint numvalues = static_cast<TqUint>( ( ( CEIL(FilterXWidth()) + 1 ) * ( CEIL(FilterYWidth()) + 1 ) ) * ( numperpixel ) );
 
     m_aFilterValues.resize( numvalues );
 
@@ -155,11 +226,11 @@ void CqBucket::InitialiseFilterValues()
     if( NULL == pFilter )
         pFilter = RiBoxFilter;
 
-    TqFloat xmax = m_XMax;
-    TqFloat ymax = m_YMax;
-    TqFloat xfwo2 = m_FilterXWidth * 0.5f;
-    TqFloat yfwo2 = m_FilterYWidth * 0.5f;
-    TqFloat xfw = m_FilterXWidth;
+    TqFloat xmax = m_DiscreteShiftX;
+    TqFloat ymax = m_DiscreteShiftY;
+    TqFloat xfwo2 = CEIL(FilterXWidth()) * 0.5f;
+    TqFloat yfwo2 = CEIL(FilterYWidth()) * 0.5f;
+    TqFloat xfw = CEIL(FilterXWidth());
 
     TqFloat subcellwidth = 1.0f / numsubpixels;
     TqFloat subcellcentre = subcellwidth * 0.5f;
@@ -176,27 +247,27 @@ void CqBucket::InitialiseFilterValues()
             TqFloat pfy = py - 0.5f;
             // Go over every subpixel in the pixel.
             TqInt sx, sy;
-            for ( sy = 0; sy < m_PixelYSamples; sy++ )
+            for ( sy = 0; sy < PixelYSamples(); sy++ )
             {
-                for ( sx = 0; sx < m_PixelXSamples; sx++ )
+                for ( sx = 0; sx < PixelXSamples(); sx++ )
                 {
                     // Get the index of the subpixel in the array
-                    TqInt sindex = index + ( ( ( sy * m_PixelXSamples ) + sx ) * numsubpixels );
-                    TqFloat sfx = static_cast<TqFloat>( sx ) / m_PixelXSamples;
-                    TqFloat sfy = static_cast<TqFloat>( sy ) / m_PixelYSamples;
+                    TqInt sindex = index + ( ( ( sy * PixelXSamples() ) + sx ) * numsubpixels );
+                    TqFloat sfx = static_cast<TqFloat>( sx ) / PixelXSamples();
+                    TqFloat sfy = static_cast<TqFloat>( sy ) / PixelYSamples();
                     // Go over each subcell in the subpixel
                     TqInt cx, cy;
-                    for ( cy = 0; cy < m_PixelXSamples; cy++ )
+                    for ( cy = 0; cy < PixelXSamples(); cy++ )
                     {
-                        for ( cx = 0; cx < m_PixelYSamples; cx++ )
+                        for ( cx = 0; cx < PixelYSamples(); cx++ )
                         {
                             // Get the index of the subpixel in the array
-                            TqInt cindex = sindex + ( ( cy * m_PixelYSamples ) + cx );
+                            TqInt cindex = sindex + ( ( cy * PixelYSamples() ) + cx );
                             TqFloat fx = ( cx * subcellwidth ) + sfx + pfx + subcellcentre;
                             TqFloat fy = ( cy * subcellwidth ) + sfy + pfy + subcellcentre;
                             TqFloat w = 0.0f;
                             if ( fx >= -xfwo2 && fy >= -yfwo2 && fx <= xfwo2 && fy <= yfwo2 )
-                                w = ( *pFilter ) ( fx, fy, m_FilterXWidth, m_FilterYWidth );
+                                w = ( *pFilter ) ( fx, fy, CEIL(FilterXWidth()), CEIL(FilterYWidth()) );
                             m_aFilterValues[ cindex ] = w;
                         }
                     }
@@ -353,17 +424,17 @@ void CqBucket::FilterBucket(TqBool empty)
     CqImagePixel * pie;
 
     TqInt datasize = QGetRenderContext()->GetOutputDataTotalSize();
-    m_aDatas.resize( datasize * Width() * Height() );
-    m_aCoverages.resize( Width() * Height() );
+    m_aDatas.resize( datasize * RealWidth() * RealHeight() );
+    m_aCoverages.resize( RealWidth() * RealHeight() );
 
-    TqInt xmax = static_cast<TqInt>( CEIL( ( FilterXWidth() - 1 ) * 0.5f ) );
-    TqInt ymax = static_cast<TqInt>( CEIL( ( FilterYWidth() - 1 ) * 0.5f ) );
-    TqFloat xfwo2 = FilterXWidth() * 0.5f;
-    TqFloat yfwo2 = FilterYWidth() * 0.5f;
-    TqInt numsubpixels = ( m_PixelXSamples * m_PixelYSamples );
+    TqInt xmax = m_DiscreteShiftX;
+    TqInt ymax = m_DiscreteShiftY;
+    TqFloat xfwo2 = CEIL(FilterXWidth()) * 0.5f;
+    TqFloat yfwo2 = CEIL(FilterYWidth()) * 0.5f;
+    TqInt numsubpixels = ( PixelXSamples() * PixelYSamples() );
 
     TqInt numperpixel = numsubpixels * numsubpixels;
-    TqInt	xlen = Width() + FilterXWidth();
+    TqInt	xlen = RealWidth();
 
     TqInt SampleCount = 0;
     CqColor imager;
@@ -371,94 +442,279 @@ void CqBucket::FilterBucket(TqBool empty)
     TqInt x, y;
     TqInt i = 0;
 
-    TqBool fImager = ( QGetRenderContext() ->optCurrent().GetStringOption( "System", "Imager" ) [ 0 ] != "null" );
+    TqBool fImager = TqFalse;
+    const CqString* systemOptions;
+    if( ( systemOptions = QGetRenderContext() ->optCurrent().GetStringOption( "System", "Imager" ) ) != 0 )
+    	if( systemOptions[ 0 ].compare("null") != 0 )
+		fImager = TqTrue;
 
     TqInt endy = YOrigin() + Height();
     TqInt endx = XOrigin() + Width();
 
-    for ( y = YOrigin(); y < endy ; y++ )
-    {
-        TqFloat ycent = y + 0.5f;
-        for ( x = XOrigin(); x < endx ; x++ )
-        {
-            TqFloat xcent = x + 0.5f;
-            TqFloat gTot = 0.0;
-            SampleCount = 0;
-            std::valarray<TqFloat> samples( 0.0f, datasize);
-
-            if(!empty)
-            {
-                TqInt fx, fy;
-                // Get the element at the upper left corner of the filter area.
-                ImageElement( x - xmax, y - ymax, pie );
-                for ( fy = -ymax; fy <= ymax; fy++ )
-                {
-                    CqImagePixel* pie2 = pie;
-                    for ( fx = -xmax; fx <= xmax; fx++ )
-                    {
-                        TqInt index = ( ( ( fy + ymax ) * FilterXWidth() ) + ( fx + xmax ) ) * numperpixel;
-                        // Now go over each subsample within the pixel
-                        TqInt sx, sy;
-                        TqInt sampleIndex = 0;
-                        for ( sy = 0; sy < m_PixelYSamples; sy++ )
-                        {
-                            for ( sx = 0; sx < m_PixelXSamples; sx++ )
-                            {
-                                TqInt sindex = index + ( ( ( sy * m_PixelXSamples ) + sx ) * numsubpixels );
-                                const SqSampleData& sampleData = pie2->SampleData( sampleIndex );
-                                CqVector2D vecS = sampleData.m_Position;
-                                vecS -= CqVector2D( xcent, ycent );
-                                if ( vecS.x() >= -xfwo2 && vecS.y() >= -yfwo2 && vecS.x() <= xfwo2 && vecS.y() <= yfwo2 )
-                                {
-                                    TqInt cindex = sindex + sampleData.m_SubCellIndex;
-                                    TqFloat g = m_aFilterValues[ cindex ];
-                                    gTot += g;
-                                    if ( pie2->Values( sx, sy ).size() > 0 )
-                                    {
-                                        SqImageSample* pSample = &pie2->Values( sx, sy ) [ 0 ];
-                                        samples += pSample->m_Data * g;
-                                        SampleCount++;
-                                    }
-                                }
-                                sampleIndex++;
-                            }
-                        }
-                        pie2++;
-                    }
-                    pie += xlen;
-                }
-            }
+	bool useSeperable = true;
 
 
-            for ( TqInt k = 0; k < datasize; k ++)
-                m_aDatas[ i*datasize + k ] = samples[k] / gTot;
+	if(!empty)
+	{
+		// non-seperable is faster for very small filter widths.
+		if(FilterXWidth() <= 1.0 || FilterYWidth() <= 1.0)
+			useSeperable = false;
 
-            // Set depth to infinity if no samples.
-            if ( SampleCount <= 0 )
-                m_aDatas[ i*datasize+6 ] = FLT_MAX;
+		if(useSeperable)
+		{
+			// seperable filter. filtering by fx,fy is equivalent to filtering
+			// by fx,1 followed by 1,fy.
 
-            if ( SampleCount >= numsubpixels)
-                m_aCoverages[ i ] = 1.0;
-            else
-                m_aCoverages[ i ] = ( TqFloat ) SampleCount / ( TqFloat ) (numsubpixels );
+			TqInt size = Width() * RealHeight() * PixelYSamples();
+			std::valarray<TqFloat> intermediateSamples( 0.0f, size * datasize);
+			std::valarray<TqInt> sampleCounts(0, size);
+			for ( y = YOrigin() - ymax; y < endy + ymax ; y++ )
+			{
+				TqFloat ycent = y + 0.5f;
+				TqInt pixelRow = (y-(YOrigin()-ymax)) * PixelYSamples();
+				for ( x = XOrigin(); x < endx ; x++ )
+				{
+					TqFloat xcent = x + 0.5f;
 
-            i++;
-        }
-    }
+					// Get the element at the left side of the filter area.
+					ImageElement( x - xmax, y, pie );
+
+					TqInt pixelIndex = pixelRow*Width() + x-XOrigin();
+
+					// filter just in x first
+					for ( TqInt sy = 0; sy < PixelYSamples(); sy++ )
+					{
+						TqFloat gTot = 0.0;
+						SampleCount = 0;
+						std::valarray<TqFloat> samples( 0.0f, datasize);
+
+						CqImagePixel* pie2 = pie;
+						for ( TqInt fx = -xmax; fx <= xmax; fx++ )
+						{
+							TqInt index = ( ( ymax * CEIL(FilterXWidth()) ) + ( fx + xmax ) ) * numperpixel;
+							// Now go over each subsample within the pixel
+							TqInt sampleIndex = sy * PixelXSamples();
+							TqInt sindex = index + ( sy * PixelXSamples() * numsubpixels );
+
+							for ( TqInt sx = 0; sx < PixelXSamples(); sx++ )
+							{
+								const SqSampleData& sampleData = pie2->SampleData( sampleIndex );
+								CqVector2D vecS = sampleData.m_Position;
+								vecS -= CqVector2D( xcent, ycent );
+								if ( vecS.x() >= -xfwo2 && vecS.y() >= -yfwo2 && vecS.x() <= xfwo2 && vecS.y() <= yfwo2 )
+								{
+									TqInt cindex = sindex + sampleData.m_SubCellIndex;
+									TqFloat g = m_aFilterValues[ cindex ];
+									gTot += g;
+									if ( pie2->OpaqueValues( sampleIndex ).m_flags & SqImageSample::Flag_Valid )
+									{
+										SqImageSample& pSample = pie2->OpaqueValues( sampleIndex );
+										for ( TqInt k = 0; k < datasize; ++k )
+											samples[k] += pSample.m_Data[k] * g;
+										sampleCounts[pixelIndex]++;
+									}
+								}
+								sampleIndex++;
+								sindex += numsubpixels;
+							}
+							pie2++;
+						}
+
+						// store the intermediate result
+						for ( TqInt k = 0; k < datasize; k ++)
+							intermediateSamples[pixelIndex*datasize + k] = samples[k] / gTot;
+
+						pixelIndex += Width();
+					}
+				}
+			}
+
+			// now filter in y.
+			for ( y = YOrigin(); y < endy ; y++ )
+			{
+				TqFloat ycent = y + 0.5f;
+				for ( x = XOrigin(); x < endx ; x++ )
+				{
+					TqFloat xcent = x + 0.5f;
+					TqFloat gTot = 0.0;
+					SampleCount = 0;
+					std::valarray<TqFloat> samples( 0.0f, datasize);
+
+					TqInt fy;
+					// Get the element at the top of the filter area.
+					ImageElement( x, y - ymax, pie );
+					for ( fy = -ymax; fy <= ymax; fy++ )
+					{
+						CqImagePixel* pie2 = pie;
+
+						TqInt index = ( ( ( fy + ymax ) * CEIL(FilterXWidth()) ) + xmax ) * numperpixel;
+						// Now go over each y subsample within the pixel
+						TqInt sx = PixelXSamples() / 2; // use the samples in the centre of the pixel.
+						TqInt sy = 0;
+						TqInt sampleIndex = sx;
+						TqInt pixelRow = (y + fy - (YOrigin()-ymax)) * PixelYSamples();
+						TqInt pixelIndex = pixelRow*Width() + x-XOrigin();
+
+						for ( sy = 0; sy < PixelYSamples(); sy++ )
+						{
+							TqInt sindex = index + ( ( ( sy * PixelXSamples() ) + sx ) * numsubpixels );
+							const SqSampleData& sampleData = pie2->SampleData( sampleIndex );
+							CqVector2D vecS = sampleData.m_Position;
+							vecS -= CqVector2D( xcent, ycent );
+							if ( vecS.x() >= -xfwo2 && vecS.y() >= -yfwo2 && vecS.x() <= xfwo2 && vecS.y() <= yfwo2 )
+							{
+								TqInt cindex = sindex + sampleData.m_SubCellIndex;
+								TqFloat g = m_aFilterValues[ cindex ];
+								gTot += g;
+								if(sampleCounts[pixelIndex] > 0)
+								{
+									SampleCount += sampleCounts[pixelIndex];
+									for ( TqInt k = 0; k < datasize; k++)
+										samples[k] += intermediateSamples[pixelIndex * datasize + k] * g;
+								}
+							}
+							sampleIndex += PixelXSamples();
+							pixelIndex += Width();
+						}
+
+						pie += xlen;
+					}
+
+					// Set depth to infinity if no samples.
+					if ( SampleCount == 0 )
+					{
+						memset(&m_aDatas[i*datasize], 0, datasize * sizeof(float));
+						m_aDatas[ i*datasize+6 ] = FLT_MAX;
+						m_aCoverages[i] = 0.0;
+					}
+					else
+					{
+						float oneOverGTot = 1.0 / gTot;
+						for ( TqInt k = 0; k < datasize; k ++)
+							m_aDatas[ i*datasize + k ] = samples[k] * oneOverGTot;
+
+						if ( SampleCount >= numsubpixels)
+							m_aCoverages[ i ] = 1.0;
+						else
+							m_aCoverages[ i ] = ( TqFloat ) SampleCount / ( TqFloat ) (numsubpixels );
+					}
+
+					i++;
+				}
+			}
+		}
+		else
+		{
+			// non-seperable filter
+			for ( y = YOrigin(); y < endy ; y++ )
+			{
+				TqFloat ycent = y + 0.5f;
+				for ( x = XOrigin(); x < endx ; x++ )
+				{
+					TqFloat xcent = x + 0.5f;
+					TqFloat gTot = 0.0;
+					SampleCount = 0;
+					std::valarray<TqFloat> samples( 0.0f, datasize);
+
+					TqInt fx, fy;
+					// Get the element at the upper left corner of the filter area.
+					ImageElement( x - xmax, y - ymax, pie );
+					for ( fy = -ymax; fy <= ymax; fy++ )
+					{
+						CqImagePixel* pie2 = pie;
+						for ( fx = -xmax; fx <= xmax; fx++ )
+						{
+							TqInt index = ( ( ( fy + ymax ) * CEIL(FilterXWidth()) ) + ( fx + xmax ) ) * numperpixel;
+							// Now go over each subsample within the pixel
+							TqInt sx, sy;
+							TqInt sampleIndex = 0;
+							for ( sy = 0; sy < PixelYSamples(); sy++ )
+							{
+								for ( sx = 0; sx < PixelXSamples(); sx++ )
+								{
+									TqInt sindex = index + ( ( ( sy * PixelXSamples() ) + sx ) * numsubpixels );
+									const SqSampleData& sampleData = pie2->SampleData( sampleIndex );
+									CqVector2D vecS = sampleData.m_Position;
+									vecS -= CqVector2D( xcent, ycent );
+									if ( vecS.x() >= -xfwo2 && vecS.y() >= -yfwo2 && vecS.x() <= xfwo2 && vecS.y() <= yfwo2 )
+									{
+										TqInt cindex = sindex + sampleData.m_SubCellIndex;
+										TqFloat g = m_aFilterValues[ cindex ];
+										gTot += g;
+										if ( pie2->OpaqueValues( sampleIndex ).m_flags & SqImageSample::Flag_Valid )
+										{
+											SqImageSample& pSample = pie2->OpaqueValues( sampleIndex );
+											for ( TqInt k = 0; k < datasize; ++k )
+												samples[k] += pSample.m_Data[k] * g;
+											SampleCount++;
+										}
+									}
+									sampleIndex++;
+								}
+							}
+							pie2++;
+						}
+						pie += xlen;
+					}
+
+
+					// Set depth to infinity if no samples.
+					if ( SampleCount == 0 )
+					{
+						memset(&m_aDatas[i*datasize], 0, datasize * sizeof(float));
+						m_aDatas[ i*datasize+6 ] = FLT_MAX;
+						m_aCoverages[i] = 0.0;
+					}
+					else
+					{
+						float oneOverGTot = 1.0 / gTot;
+						for ( TqInt k = 0; k < datasize; k ++)
+							m_aDatas[ i*datasize + k ] = samples[k] * oneOverGTot;
+
+						if ( SampleCount >= numsubpixels)
+							m_aCoverages[ i ] = 1.0;
+						else
+							m_aCoverages[ i ] = ( TqFloat ) SampleCount / ( TqFloat ) (numsubpixels );
+					}
+
+					i++;
+				}
+			}
+		}
+	}
+	else
+	{
+		// empty bucket.
+		TqInt size = Width()*Height();
+		memset(&m_aDatas[0], 0, size * datasize * sizeof(float));
+		memset(&m_aCoverages[0], 0, size * sizeof(float));
+		for(i = 0; i<size; ++i)
+		{
+			// Set the depth to infinity.
+			m_aDatas[ i*datasize+6 ] = FLT_MAX;
+		}
+	}
 
     i = 0;
     ImageElement( XOrigin(), YOrigin(), pie );
     endy = Height();
     endx = Width();
 
+	// Set the coverage and alpha values for the pixel.
     for ( y = 0; y < endy; y++ )
     {
         CqImagePixel* pie2 = pie;
         for ( x = 0; x < endx; x++ )
         {
-            for (TqInt k=0; k < datasize; k++)
-                pie2->GetPixelSample().m_Data[k] = m_aDatas[ i * datasize + k ];
-            pie2->SetCoverage( m_aCoverages[ i++ ] );
+            SqImageSample& spl = pie2->GetPixelSample();
+			for (TqInt k=0; k < datasize; k++)
+                spl.m_Data[k] = m_aDatas[ i * datasize + k ];
+            spl.SetCoverage( m_aCoverages[ i++ ] );
+	
+			// Calculate the alpha as the combination of the opacity and the coverage.
+			TqFloat a = ( spl.Os()[0] + spl.Os()[1] + spl.Os()[2] ) / 3.0f;
+			pie2->SetAlpha(a * spl.Coverage());
+
             pie2++;
         }
         pie += xlen;
@@ -467,7 +723,7 @@ void CqBucket::FilterBucket(TqBool empty)
     endy = YOrigin() + Height();
     endx = XOrigin() + Width();
 
-    if ( NULL != QGetRenderContext() ->optCurrent().pshadImager() && NULL != QGetRenderContext() ->optCurrent().pshadImager() ->pShader() )
+    if ( NULL != QGetRenderContext() ->optCurrent().pshadImager() && QGetRenderContext() ->optCurrent().pshadImager() ->pShader() )
     {
         QGetRenderContext() ->Stats().MakeFilterBucket().Stop();
         // Init & Execute the imager shader
@@ -524,7 +780,7 @@ void CqBucket::ExposeBucket()
         TqInt   nextx;
         endy = Height();
         endx = Width();
-        nextx = Width() + FilterXWidth();
+        nextx = RealWidth();
 
         for ( y = 0; y < endy; y++ )
         {
@@ -563,7 +819,7 @@ void CqBucket::QuantizeBucket()
     TqInt   nextx;
     endy = Height();
     endx = Width();
-    nextx = Width() + FilterXWidth();
+    nextx = RealWidth();
 
 
     if ( QGetRenderContext() ->optCurrent().GetIntegerOption( "System", "DisplayMode" ) [ 0 ] & ModeRGB )
@@ -587,23 +843,26 @@ void CqBucket::QuantizeBucket()
             CqImagePixel* pie2 = pie;
             for ( x = 0; x < endx; x++ )
             {
-                double r, g, b;
+                double r, g, b, a;
                 double _or, _og, _ob;
                 double s = random.RandomFloat();
                 CqColor col = pie2->Color();
                 CqColor opa = pie2->Opacity();
+				TqFloat alpha = pie2->Alpha();
                 if ( modf( one * col.fRed () + ditheramplitude * s, &r ) > 0.5 ) r += 1;
                 if ( modf( one * col.fGreen() + ditheramplitude * s, &g ) > 0.5 ) g += 1;
                 if ( modf( one * col.fBlue () + ditheramplitude * s, &b ) > 0.5 ) b += 1;
                 if ( modf( one * opa.fRed () + ditheramplitude * s, &_or ) > 0.5 ) _or += 1;
                 if ( modf( one * opa.fGreen() + ditheramplitude * s, &_og ) > 0.5 ) _og += 1;
                 if ( modf( one * opa.fBlue () + ditheramplitude * s, &_ob ) > 0.5 ) _ob += 1;
+                if ( modf( one * alpha + ditheramplitude * s, &a ) > 0.5 ) a += 1;
                 r = CLAMP( r, min, max );
                 g = CLAMP( g, min, max );
                 b = CLAMP( b, min, max );
                 _or = CLAMP( _or, min, max );
                 _og = CLAMP( _og, min, max );
                 _ob = CLAMP( _ob, min, max );
+                a = CLAMP( a, min, max );
                 col.SetfRed ( r );
                 col.SetfGreen( g );
                 col.SetfBlue ( b );
@@ -612,7 +871,7 @@ void CqBucket::QuantizeBucket()
                 opa.SetfBlue ( _ob );
                 pie2->SetColor( col );
                 pie2->SetOpacity( opa );
-
+				pie2->SetAlpha( a );
                 pie2++;
             }
             pie += nextx;
@@ -701,6 +960,7 @@ void CqBucket::ShutdownBucket()
 	m_aSamplePositions.clear();
 }
 
+
 //---------------------------------------------------------------------
 
 END_NAMESPACE( Aqsis )
