@@ -213,6 +213,9 @@ CqOptions& CqOptions::operator=( const CqOptions& From )
     m_bScreenWindowCalled = From.m_bScreenWindowCalled;
     m_bFormatCalled = From.m_bFormatCalled;
 
+    m_funcFilter = From.m_funcFilter;
+    m_pshadImager = From.m_pshadImager;
+
     DeleteImager();
 
     // Copy the system options.
@@ -599,9 +602,9 @@ void CqOptions::LoadImager( const CqString& strName )
 {
     DeleteImager();
 
-    IqShader* pShader = static_cast<IqShader*>( QGetRenderContext() ->CreateShader( strName.c_str(), Type_Imager ) );
+    boost::shared_ptr<IqShader> pShader = QGetRenderContext() ->CreateShader( strName.c_str(), Type_Imager );
 
-    if ( pShader == 0 ) return ;
+    if ( !pShader ) return ;
 
     m_pshadImager = new CqImagersource( pShader, RI_TRUE );
     m_pshadImager->pShader() ->PrepareDefArgs();
